@@ -1,4 +1,4 @@
-# Copyright (c) 2010-2017 Trevor L. Davis <trevor.l.davis@gmail.com>  
+# Copyright (c) 2010-2019 Trevor L. Davis <trevor.l.davis@gmail.com>  
 # Copyright (c) 2015 Rick FitzJohn https://github.com/richfitz
 # Copyright (c) 2013 Kirill Müller https://github.com/krlmlr
 # Copyright (c) 2011 Jim Nikelski <nikelski@bic.mni.mcgill.ca>
@@ -227,11 +227,18 @@ OptionParser <- function(usage = "usage: %prog [options]", option_list=list(),
 #'        help="Standard deviation if generator == \"rnorm\" [default %default]")
 #'
 #' @export
-make_option <- function(opt_str, action="store", type=NULL, dest=NULL, default=NULL, help="", metavar=NULL, callback=NULL, callback_args=NULL) {
+make_option <- function(opt_str, action="store", type=NULL, dest=NULL, default=NULL, 
+                        help="", metavar=NULL, callback=NULL, callback_args=NULL) {
 
     # flags
     short_flag <- opt_str[grepl("^-[[:alpha:]]", opt_str)]
-    if(length(short_flag)) {} else { short_flag <- as.character(NA) }
+    if(length(short_flag) == 0) {
+        short_flag <- NA_character_ 
+    } else {
+        if (nchar(short_flag) > 2) {
+            stop(paste("Short flag", short_flag, "must only be a '-' and a single letter"))
+        }
+    }
     long_flag <- opt_str[grepl("^--[[:alpha:]]", opt_str)]
     if(length(long_flag)) {} else {stop("We require a long flag option")}
 
@@ -259,7 +266,7 @@ make_option <- function(opt_str, action="store", type=NULL, dest=NULL, default=N
     if(is.null(dest)) { dest <- sub("^--", "", long_flag) }
     # metavar
     if(is.null(metavar)) {
-        if(action == "store") { 
+        if(.option_needs_argument_helper(action, type)) { 
             metavar <- sub("^--", "", long_flag)
         } else {
             metavar <- character(0)
@@ -324,14 +331,14 @@ print_help <- function(object) {
         cat("\t")
         if(!is.na(option@short_flag)) {
             cat(option@short_flag)
-            if( option@action == "store" ) {
+            if (.option_needs_argument(option)) {
                 cat(" ", toupper(option@metavar), sep="")
             }
             cat(", ")
         }
         if(!is.null(option@long_flag)) {
             cat(option@long_flag)
-            if( option@action == "store" ) {
+            if(.option_needs_argument(option)) {
                 cat("=", toupper(option@metavar), sep="")
             }
         }
@@ -580,7 +587,7 @@ parse_args2 <- function(object, args = commandArgs(trailingOnly = TRUE),
             for (ii in seq_along(object@options)) {
                 option <- object@options[[ii]]
                 if(option@long_flag == argument)
-                    return(option@action == "store") 
+                    return(.option_needs_argument(option)) 
             }
         }
     } else { # is a short flag
@@ -588,7 +595,7 @@ parse_args2 <- function(object, args = commandArgs(trailingOnly = TRUE),
         for (ii in seq_along(object@options)) {
             option <- object@options[[ii]]
             if(!is.na(option@short_flag) && option@short_flag == last_flag)
-                return(option@action == "store") 
+                return(.option_needs_argument(option)) 
         }
     }
 }
@@ -631,7 +638,7 @@ parse_args2 <- function(object, args = commandArgs(trailingOnly = TRUE),
         option <- object@options[[ii]]
         option_strings <- c(option_strings, option@short_flag)
         option_strings <- c(option_strings, option@long_flag)
-        if (option@action == "store") {
+        if (.option_needs_argument(option)) {
             n_arguments <- c(n_arguments, 1, 1)
         } else {
             n_arguments <- c(n_arguments, 0, 0)
@@ -644,11 +651,23 @@ parse_args2 <- function(object, args = commandArgs(trailingOnly = TRUE),
 .convert_to_getopt <- function(object) {
     short_flag <- sub("^-", "", object@short_flag)
     long_flag <- sub("^--", "", object@long_flag)
-    if( object@action %in% c("store_true", "store_false") ) {
-        argument <- 0
-    } else {
-        argument <- 1
-    }
+    argument <- ifelse(.option_needs_argument(object), 1, 0)
     return( c( long_flag, short_flag, argument, object@type, object@help) )
 }
 
+.option_needs_argument <- function(option) {
+    .option_needs_argument_helper(option@action, option@type)
+}
+.option_needs_argument_helper <- function(action, type) {
+    if (action == "store") {
+        TRUE
+    } else if (action == "callback") {
+        if (is.null(type)) {
+            FALSE
+        } else {
+            TRUE
+        }
+    } else {
+        FALSE
+    }
+}
