@@ -1,4 +1,4 @@
-# Copyright 2010-2017 Trevor L Davis <trevor.l.davis@gmail.com>
+# Copyright 2010-2019 Trevor L Davis <trevor.l.davis@gmail.com>
 # Copyright 2013 Kirill Müller
 # Copyright 2008 Allen Day
 #  
@@ -40,6 +40,7 @@ test_that("make_option works as expected", {
                 make_option("--logical", default=TRUE))
     expect_equal(make_option("--filename")@type, "character")
     expect_that(make_option("badflag"), throws_error())
+    expect_error(make_option("-cd"), "must only be")
 })
 
 get_long_flags <- function(parser) {
@@ -137,6 +138,11 @@ test_that("callback works as expected", {
                          action="callback", help="Squared distance between two points", callback=callback_fn, callback_args=list(2))
     opts <- parse_args(parser1, args = c("--squared_distance=16"))
     expect_equal(opts$squared_distance, 256)
+    # Bug found by Ni Huang (#28)
+    opts <- parse_args(parser1, positional_argument=TRUE, args=c("-s", "3"))
+    expect_equal(opts$options$squared_distance, 9)
+    # Bug found by Ni Huang (#29)
+    expect_output(print_help(parser1), "SQUARED_DISTANCE")
 
     parser2 <- add_option(parser0, c("-v", "--value"), type="integer",
                          action="callback", callback=callback_fn, callback_args=list(n=3))
@@ -152,6 +158,8 @@ test_that("callback works as expected", {
     expect_warning(add_option(parser0, "--warning", callback_args=list(3,b=4)), "callback_args argument is supplied for non-callback action")
     expect_error(add_option(parser0, "--warning", action="callback", callback="hello"), "Option type cannot be inferred")
     expect_warning(add_option(parser0, "--warning", action="callback", type="numeric", callback="hello"), "callback argument is not a function")
+
+
 })
 
 # Bug found by Miroslav Posta
